CREATE TABLE IF NOT EXISTS `acl_user_google` ( 
`id_acl_user_google` int(11) NOT NULL AUTO_INCREMENT,
`id_acl_user` int(11) NOT NULL,
`google_user_id` varchar(255) CHARACTER SET utf8, 
PRIMARY KEY (`id_acl_user_google`) 
) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;