<?php

if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

class auth_google_admin extends MX_Controller
{

    public function manage()
    {
        $this->load->library('ciext/configurator');
        $conf = new configurator('auth_google');
        $conf->add_input('redirect_link');
        $conf->add_textarea('json_key');

        $conf->display_as('redirect_link', $this->lang->line('Redirect link'));
        $conf->display_as('json_key', $this->lang->line('Google JSON key'));

        return array('content' => $this->getExplains() . $conf->render(), 'title' => $this->lang->line('Auth by google'));
    }

    protected function getExplains()
    {
        return '<div style="width:100%;float:left;margin-bottom:15px;padding: 10px;border: 1px solid #333;">' . $this->lang->line('Инструкция!<br>Перейдите по <a href="https://code.google.com/apis/console/" target="_blank">ССЫЛКЕ</a> и создайте проект, где предоставьте права на "Contacts API". В меню "Учетные данные" вы сможете настроить ссылки для переадрисации, а также сможете скачать json ключ.') . '</div>';
    }

}
